#import "MyOpenGLView.h"
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import "NTBox.h"

#define MAX_CONTACTS 4		// maximum number of contact points per body

dWorldID world;
dSpaceID space;
dJointGroupID contactgroup;

static void nearCallback (void *data, dGeomID o1, dGeomID o2)
{
  int i;
  // if (o1->body && o2->body) return;

  // exit without doing anything if the two bodies are connected by a joint
  dBodyID b1 = dGeomGetBody(o1);
  dBodyID b2 = dGeomGetBody(o2);
  if (b1 && b2 && dAreConnectedExcluding (b1,b2,dJointTypeContact)) return;

  dContact contact[MAX_CONTACTS];   // up to MAX_CONTACTS contacts per box-box
  for (i=0; i<MAX_CONTACTS; i++)
  {
    contact[i].surface.mode = dContactBounce | dContactSoftCFM;
    contact[i].surface.mu = dInfinity;
    contact[i].surface.mu2 = 0;
    contact[i].surface.bounce = 0.1;
    contact[i].surface.bounce_vel = 0.1;
    contact[i].surface.soft_cfm = 0.01;
  }
  int numc;
  if (numc = dCollide (o1,o2,MAX_CONTACTS,&contact[0].geom, sizeof(dContact)))
  {
    dMatrix3 RI;
    dRSetIdentity (RI);
    const dReal ss[3] = {0.02,0.02,0.02};
    for (i=0; i<numc; i++)
	{
      dJointID c = dJointCreateContact (world,contactgroup,contact+i);
      dJointAttach (c,b1,b2);
    }
  }
}

@implementation MyOpenGLView

- (id)initWithFrame:(NSRect)frameRect
{
	NSOpenGLPixelFormatAttribute		attributes[]={
		NSOpenGLPFADepthSize, 16,
		NSOpenGLPFAAccelerated, 1,
		nil};

    // This next line calls our parent's init routine
    self = [super initWithFrame:frameRect pixelFormat:[[[NSOpenGLPixelFormat alloc] initWithAttributes:attributes] autorelease]];
    
    [self setup];
	
	// start timer
	worldTime = [NSDate timeIntervalSinceReferenceDate];
	[[NSTimer scheduledTimerWithTimeInterval: 0.0001 target: self selector:@selector(doTimeStep:) userInfo: nil repeats: YES] retain];
    
    return self;
}
	
- (void)setup
{
	[self setupGL];
	[self initScene];
}

- (void)dealloc
{
	// destroy the Newton world
	dJointGroupDestroy (contactgroup);
	dSpaceDestroy (space);
	dWorldDestroy (world);
}

#pragma mark Physics

- (void)initScene
{
	// create world
	world = dWorldCreate();
	space = dHashSpaceCreate (0);
	contactgroup = dJointGroupCreate (0);
	dWorldSetGravity (world, 0, -9.8, 0);
	dWorldSetCFM (world, 1e-5);
	dCreatePlane (space,0,1,0,-5);	
	
	// create 100 stacks of 10 boxes each
	int i, j, k;
	boxes = [[NSMutableArray alloc] init];
	
	for (k = 0; k < 10; k ++)
	{ 
		for (j = 0; j < 10; j ++)
		{ 
			for (i = 0; i < 10; i ++)
			{
				// create a graphic box
				NTBox *box = [[NTBox alloc] initWithWorld:world inSpace:space atX:-10+k*2 atY:i*2 atZ:2+j*4];
				[boxes addObject:box];
			}
		}
	}
}

#pragma mark Graphics

- (void)setupGL
{
	// initilize opengl	
    [[self openGLContext] makeCurrentContext];
	
	// set the background color
	glClearColor(0.29, 0.42, 0.58, 0.0);
	
	// set up a single white light
	GLfloat lightColor[] = { 0.7f, 0.7f, 0.7f, 0.0 };
	GLfloat lightAmbientColor[] = { 0.125f, 0.125f, 0.125f, 0.0 };
	GLfloat lightPoition[] = { 00.0f, 200.0f, 500.0f, 0.0 };
	
	glLightfv(GL_LIGHT0, GL_POSITION, lightPoition);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbientColor);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	
	//enable depth testing
    glEnable(GL_DEPTH_TEST);
}

- (void) doTimeStep:(id)sender
{
	[self setNeedsDisplay:YES];
}

- (void)reshape
{
	NSRect bounds = [self bounds];
	
	glViewport(0, 0, (GLsizei) bounds.size.width, (GLsizei) bounds.size.height);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0, (GLfloat) bounds.size.width/(GLfloat) bounds.size.height, 1.0, 100.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -50);
}

- (void)drawRect:(NSRect)r
{
	static float fps = 100;
	
	// updtate the Newton physics world
	NSTimeInterval realTime = [NSDate timeIntervalSinceReferenceDate];
	if([physicsSwitch state] == NSOnState)
	{
		dSpaceCollide (space, 0, &nearCallback);
		dWorldStep (world, 0.02);//realTime-worldTime);

		// remove all contact joints
		dJointGroupEmpty (contactgroup);
	}
	
	fps += (1/(realTime-worldTime)-fps)/10;
	[fpsTextField performSelectorOnMainThread:@selector(setStringValue:) withObject:[NSString stringWithFormat:@"%0.2f fps", fps] waitUntilDone:NO];
	
	worldTime = realTime;

	// render the boxes
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	//draw floor
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
	glNormal3f(0.0, 1.0, 0.0);
	glVertex3f(-200,  -5,  200);
	glVertex3f( 200,  -5,  200);
	glVertex3f( 200,  -5, -200);
	glVertex3f(-200,  -5, -200);
	glEnd();

	[boxes makeObjectsPerformSelector:@selector(draw)];
	
	glFlush();
}

@end
